#Script for the article 
#Using External Information for More Precise Inferences in General Regression Models
#Part 1: Description and definition of the function that can be used for applications
#Part 2: Constants and Simulation study
#Part 3: Analysis of the data of Pluck & Ruales-Chieruzzi (2021)


########Part 1: The Function########

#To facilitate the applicability of our results, only one function 
#(interval_gmm) is required to compute the estimates, 
#their standard deviations and confidence interval unions.
#We will explain its use first.

##Description:

#Arguments:
#interval_gmm(X,y,mom,I_min,I_max,grid_step,intercept)

#The arguments X and y refer to the matrix containing the 
#independent variables (X) and the vector containing the 
#dependent variable (y). 

#The argument mom specifies which external moment
#to use in the analysis. It should be a character 
#vector where each entry has the following form: 
#"EXi","EY","EXiY","CXiY","RXiY","BXiY","EY2","VY"
#i is a placeholder for an integer, indicating the 
#position of the selected independent variable in 
# the matrix X, e.g., EX1 indicates that the expected 
#value of the first variable (first column) of X is known.

#I_min and I_max should be vectors containing the lower and upper
#bounds of the external intervals. The default is I_max=I_min,
#so if only I_min is provided, the function will 
#calculate the results for the external point values I_min.

#grid_points specifies how many grid points should
#be generated FOR EACH DIMENSION of the external interval 
#to approximate the estimate and variance intervals, 
#as well as the confidence union. The default is 10001 points. 
#Note that for n moments, the number of grid points is 10001^n,
#so the time required to compute the intervals is exponential 
#in the number of moments.

#Finally, the intercept flag indicates whether the model should
#have an intercept (TRUE) or not (FALSE). The default is TRUE.

#The output will look like the following:

#     beta_ug   beta_og      s_ug      s_og     int_ug   int_og
#[1,] 0.4368288 0.7643962 0.7072362 0.7248343 -1.1041065 2.343675
#[2,] 0.5909356 0.5909356 0.3488715 0.3578154 -0.1886772 1.370548
#[3,] 1.9990596 1.9990596 1.2971703 1.3304256 -0.8996888 4.897808

#The columns beta_ug, beta_og contain the lower and upper bounds
#of the externally informed GMM-estimate, s_ug and s_og the bounds of
#its standard deviations, and int_ug, int_og the bounds of the
#confidence interval union.
#The rows are ordered like the columns of X, [1,] is the
#intercept (if not set to FALSE), [2,] is the slope for the first 
#independent variable (first column of X), [3,] is the slope for the
#second (second column of X), and so on.

##Definition:
interval_gmm<-function(X,y,mom,I_min,I_max=I_min,grid_points=10001,intercept=TRUE){

######internal functions
#calculate external moment equations per unit
calc_h<-function(X,y,E,mom,positions){
  h<-c()
  for (i_h in 1:length(mom)) {
    h<-cbind(h,switch(mom[i_h],
           EX= X[,positions[i_h]]-E[i_h],
           EY = y-E[i_h],
           EXY = X[,positions[i_h]]*y-E[i_h],
           EY2 = y^2-E[i_h],
           VY = (y-mean(y))^2-E[i_h],
           CXY = (y-mean(y))*(X[,positions[i_h]]-mean(X[,positions[i_h]]))-E[i_h],
           RXY = (y-mean(y))*(X[,positions[i_h]]-mean(X[,positions[i_h]]))/(sd(y)*sd(X[,positions[i_h]]))-E[i_h],
           BXY = (y-mean(y))*(X[,positions[i_h]]-mean(X[,positions[i_h]]))/var(X[,positions[i_h]])-E[i_h]
           )
        )
    }
  return(h)
}

##caculate (X'X)^-1 Omega_R matrix 
#See supplementary material II, Table 6, 
#replace expected values by means
#and sigma by sigma hat 
#(See Section 5 on how sigma hat is computed in
#two steps, first using beta_OLS then using beta_GMM)

calc_Omega_R_X<-function(X,y,sigma,mom,beta_ols,positions){
  Omega_R<-c()
  for (i_o in 1:length(mom)) {
    Omega_R<-cbind(Omega_R,switch(mom[i_o],
                                  EX= rep(0,ncol(X)),
                                  EY = c(1,rep(0,ncol(X)-1)),
                                  EXY = diag(ncol(X))[,positions[i_o]],
                                  EY2 = 2*beta_ols,
                                  VY = 2*(beta_ols-c(mean(y),rep(0,ncol(X)-1))),
                                  CXY = c(-mean(X[,positions[i_o]]),rep(0,ncol(X)-1))+diag(ncol(X))[,positions[i_o]],
                                  RXY = (c(-mean(X[,positions[i_o]]),rep(0,ncol(X)-1))+diag(ncol(X))[,positions[i_o]])/(sd(y)*sd(X[,positions[i_o]])),
                                  BXY = (c(-mean(X[,positions[i_o]]),rep(0,ncol(X)-1))+diag(ncol(X))[,positions[i_o]])/var(X[,positions[i_o]])
    )
    )
  }
  return(sigma^2*Omega_R/nrow(X)) #dividing by n because (X'X)^-1 = 1/n*(X'X/n)^-1 -> 1/n E(xx')^-1
}

#calculate Omega_h

#single valued omega_h are determined by assuming epsilon ~ N(0,sigma^2),
#see supplementary material II for an example.
#=> set E(epsilon) and E(epsilon^3) to 0
#and E(epsilon^4)= 3 * sigma^4
#=> then replace all expected value by means

#Using multiple moments simultaneously, 
#E(epsilon) and E(epsilon^3) are NOT set to 0.

calc_Omega_h<-function(X,y,E,mom,sigma,beta_ols_h,positions){
  h_d<-calc_h(X,y,E,mom,positions)
  if(length(mom)==1){ 
    Omega_h<-switch(mom,
                    EX= 1, #Placeholder as EX = OLS
                    EY = mean((X%*%beta_ols_h-E)^2)+sigma^2,
                    EXY = mean((X[,positions]*X%*%beta_ols_h-E)^2)+sigma^2*mean(X[,positions]^2),
                    EY2 = mean(((X%*%beta_ols_h)^2-E)^2+2*sigma^2*((X%*%beta_ols_h)^2-E)+3*sigma^4+4*sigma^2*(X%*%beta_ols_h)^2),
                    VY = mean(((X%*%beta_ols_h-mean(X%*%beta_ols_h))^2-E)^2+2*sigma^2*((X%*%beta_ols_h-mean(X%*%beta_ols_h))^2-E)+3*sigma^4+4*sigma^2*(X%*%beta_ols_h-mean(X%*%beta_ols_h))^2),
                    CXY = mean(((X%*%beta_ols_h-mean(X%*%beta_ols_h))*(X[,positions]-mean (X[,positions]))-E)^2)+sigma^2*var(X[,positions]),
                    RXY = (mean(((X%*%beta_ols_h-mean(X%*%beta_ols_h))*(X[,positions]-mean(X[,positions]))-E*sd(y)*sd(X[,positions]))^2)+sigma^2*var(X[,positions]))/var(y)/var(X[,positions]),
                    BXY = (mean(((X%*%beta_ols_h-mean(X%*%beta_ols_h))*(X[,positions]-mean(X[,positions]))-E*var(X[,positions]))^2)+sigma^2*var(X[,positions]))/var(X[,positions])/var(X[,positions])
    )
  }else{Omega_h<-t(h_d)%*%h_d/nrow(X)} #multiple moments
  return(Omega_h)
}

#calculate gmm-parameter-estimate (see Theorem 1)
calc_beta_gmm<-function(X,y,E,mom,Omega_R_X,Omega_h,positions){
  h<-calc_h(X,y,E,mom,positions)
  solve(t(X)%*%X)%*%t(X)%*%y-Omega_R_X%*%solve(Omega_h)%*%t(h)%*%rep(1,nrow(X))
  }

#calculate gmm variance estimate (see Theorem 1, replace expected values by means)
calc_var_gmm<-function(X,y,Omega_R_X,Omega_h,sigma){
  diag(sigma^2*solve(t(X)%*%X)-nrow(X)*Omega_R_X%*%solve(Omega_h)%*%t(Omega_R_X))
  #Note that E(xx') is substituted by (X'X)/n => times nrow, not devided by nrow as in Theorem 1
}

###########

### core of interval_gmm():
positions<-as.numeric(gsub("\\D","",mom)) #extract numbers of variables
if(intercept==T){positions<-positions+1}
mom[mom!="EY2"]<-gsub("[[:digit:]]+", "", mom[mom!="EY2"]) #reduce mom to cases

#calculate bounds for estimator intervals, standard deviation intervals
#and confidence interval unions

#init
beta_ols_h<-as.vector(lm(y~X)$coefficients)
if(intercept==TRUE){X<-cbind(1,X)} #changing X to include column of 1
  int_ug<-rep(Inf,ncol(X))
  int_og<-rep(-Inf,ncol(X))
  beta_ug<-rep(Inf,ncol(X))
  beta_og<-rep(-Inf,ncol(X))
  var_ug<-rep(Inf,ncol(X))
  var_og<-rep(-Inf,ncol(X))
#build grid
  g_l<-rep(list(0:(grid_points-1)),sum(I_max!=I_min))
  grid<-expand.grid(g_l)
  for (i_Ki in 1:(grid_points^sum(I_max!=I_min))) {
      E_I<-I_min
      #traversing grid points for intervals (no grid required for point information)
      E_I[I_max!=I_min]<-I_min[I_max!=I_min]+as.numeric(grid[i_Ki,])*(I_max-I_min)[I_max!=I_min]/(grid_points-1)
      ### calculate gmm-estimations for grid-point
      #first step
      sigma<-summary(lm(y~X[,-1]))[[6]]
      Omega_R<-calc_Omega_R_X(X,y,sigma,mom,beta_ols_h,positions)
      Omega_h<-calc_Omega_h(X,y,E_I,mom,sigma,beta_ols_h,positions)
      #second step
      beta_gmm_dummy<-calc_beta_gmm(X,y,E_I,mom,Omega_R,Omega_h,positions)
      sigma<-sqrt(sum((y-X%*%beta_gmm_dummy)^2)/(nrow(X)-length(beta)))
      Omega_R<-calc_Omega_R_X(X,y,sigma,mom,beta_gmm_dummy,positions)
      Omega_h<-calc_Omega_h(X,y,E_I,mom,sigma,beta_gmm_dummy,positions)
      beta_gmm<-calc_beta_gmm(X,y,E_I,mom,Omega_R,Omega_h,positions)
      var_gmm<-calc_var_gmm(X,y,Omega_R,Omega_h,sigma)
      #calculate new confidence interval
      new_ug<-beta_gmm-qt(0.975,df=nrow(X)-ncol(X))*sqrt(var_gmm)
      new_og<-beta_gmm+qt(0.975,df=nrow(X)-ncol(X))*sqrt(var_gmm)
      #update lower bounds for confidence union
      if(sum(new_ug<int_ug)>0){
        int_ug[which(new_ug<int_ug)]<-new_ug[which(new_ug<int_ug)]
      }
      #update upper bounds for confidence union
      if(sum(new_og>int_og)>0){
        int_og[which(new_og>int_og)]<-new_og[which(new_og>int_og)]
      }
      #update lower bounds for beta estimate and its variance estimate
      if(sum(beta_gmm<beta_ug)>0){
        beta_ug[which(beta_gmm<beta_ug)]<-beta_gmm[which(beta_gmm<beta_ug)]
      }
      if(sum(var_gmm<var_ug)>0){
        var_ug[which(var_gmm<var_ug)]<-var_gmm[which(var_gmm<var_ug)]
      }
      #update upper bounds for for beta estimate and its variance estimate
      if(sum(beta_gmm>beta_og)>0){
        beta_og[which(beta_gmm>beta_og)]<-beta_gmm[which(beta_gmm>beta_og)]
      }
      if(sum(var_gmm>var_og)>0){
        var_og[which(var_gmm>var_og)]<-var_gmm[which(var_gmm>var_og)]
      }
  }
  return(cbind(beta_ug,beta_og,s_ug=sqrt(var_ug),s_og=sqrt(var_og),int_ug,int_og))
}

########Constants#########

#NOTE: The constants represent only one simulation scenario 
#and must be varied accordingly to reproduce all the results 
# reported in the paper.

n=15 #sample size (15,30,50 and 100 in Section 5)
nsim=500 #number of simulations
grid_points=101 #grid size for estimate intervals and CI union
I_rad=0.1 #radius of moment interval relative to point value 
#0.1 for correctly specified setting, 0.3 for misspecified setting (Section 5)
misspecified=F #F for correctly specified setting, T for misspecified
if(misspecified==T){I_rad<-0.3} #Change to 0.3 for misspecified setting

##Data Generation: 
# X1 is normal-distributed, X2 is bernoulli-distributed
# X1 independent of X2, both drawn iid
# Y = beta_0 + X1 beta_1 + X2 beta_2 + e
# e is normal distributed with mean 0 OR linearly transformed chi^2 (Section 5)

###moments in the setting
# E = Expected value, V = Variance, B = Beta, C = Covariance, R = Correlation

EX=2 
VX=4
EX2=0.4
VX2=EX2*(1-EX2) # bernoulli variance
beta=c(1,0.5,2)
BXY=beta[2]
BX2Y=beta[3]
e_var=9 # error variance
EY=as.vector(t(beta)%*%c(1,EX,EX2))
VY=as.vector(t(beta[-1]^2)%*%c(VX,VX2)+e_var)
EY2=VY+EY^2 # Expected value of Y^2
CXY=BXY*VX
EXY=CXY+EX*EY
RXY=CXY/sqrt(VX*VY)
CX2Y=BX2Y*VX2
EX2Y=CX2Y+EX2*EY
RX2Y=CX2Y/sqrt(VX2*VY)

########Simulations########

#init
erg<-c()
reduction<-c()
all_mom<-c("EX1","EY","EX1Y","CX1Y","RX1Y","BX1Y","EX2Y","CX2Y","RX2Y","BX2Y","EY2","VY")
all_E<-c(EX,EY,EXY,CXY,RXY,BXY,EX2Y,CX2Y,RX2Y,BX2Y,EY2,VY)
if(misspecified==T){all_E<-all_E*(1+I_rad)}

#loop traversing all single moments
for (j in 1:length(all_mom)) {
  #init per moment
  mom<-all_mom[j]
  E<-all_E[j]
  ug<-c()
  og<-c()
  v_red<-c()
  beta_gmm<-c()
  var_gmm<-c()
  coverage_int<-c(0,0,0)
  length_interval_KI<-c()
#loop calculates results for moment
for(i in 1:nsim){
  ### data generation
  #random X 
  set.seed(i)
  X<-cbind(rnorm(n,mean=EX,sd=sqrt(VX)),rbinom(n,size=1,prob=EX2))
  #for small n: control for perfect linear dependencies
  while(sum(X[,2])==n | sum(X[,2])==0){X<-cbind(rnorm(n,mean=EX,sd=sqrt(VX)),rbinom(n,size=1,prob=EX2))}
  
  if(misspecified==F){
  y<-cbind(1,X)%*%beta+rnorm(n,mean = 0,sd=sqrt(e_var))}else{
  y<-cbind(1,X)%*%beta+(rchisq(n,df=1)-1)/sqrt(2)*sqrt(e_var)
  }
  
  ##computation for the point value (=> no large grid required)
  loop_erg<-interval_gmm(X,y,mom,I_min=E,grid_points=2)
  
  beta_gmm<-cbind(beta_gmm,loop_erg[,1])
  var_gmm<-cbind(var_gmm,loop_erg[,3]^2)
  ## calculate relative variance reduction and confidence interval bounds
  sigma<-summary(lm(y~X))[[6]]
  v_red<-cbind(v_red,(1-var_gmm[,i]/diag(sigma^2*solve(t(cbind(1,X))%*%cbind(1,X)))))
  ug<-cbind(ug,loop_erg[,5])
  og<-cbind(og,loop_erg[,6])
  ### calculate coverage and CI-union for moment interval-value
  I_min=(1-I_rad)*E 
  I_max=(1+I_rad)*E
  KI_int<-interval_gmm(X,y,mom,I_min,I_max,grid_points=grid_points)[,c(5,6)]
  coverage_int<-coverage_int+(KI_int[,1]<beta & beta< KI_int[,2])/nsim
  length_interval_KI<-cbind(length_interval_KI,KI_int[,2]-KI_int[,1])
  }
  ### combining results from single moments
  coverage<-c(sum(og[1,]>beta[1]&ug[1,]<beta[1])/nsim,
  sum(og[2,]>beta[2] & ug[2,]< beta[2])/nsim,
  sum(og[3,]>beta[3] & ug[3,]< beta[3])/nsim)
  KI_length_point<-apply(og-ug,MARGIN = 1,FUN=mean)
  KI_length_interval<-apply(length_interval_KI,MARGIN = 1,FUN=mean)
  erg<-c(erg,list(cbind(estim_b=apply(beta_gmm,1,mean),
                        var_b=apply(beta_gmm,1,var),
                        mean_var=apply(var_gmm,1,mean),
                        coverage,coverage_int,
                        KI_length_point,KI_length_interval)))
  reduction<-c(reduction,list(v_red))
}

#results 
lapply(erg,function(x){round(x,digits=3)})
#list number = moment number 
#(first matrix is for EX1, second for EY and so on)

#columns: 
#1. mean of point gmm-estimates
#2. variance of point gmm-estimates
#3. mean of point gmm-variance estimates
#4. coverage for point gmm-estimation
#5. coverage for interval gmm-estimation
#6. length of CI for point gmm-estimation
#7. length of CI-union for interval gmm-estimation

#mean of the relative variance reduction per case
for(i in 1:12){print(apply(reduction[[i]],MARGIN=1,mean))}
#order: i-th row = i-th moment ; j-th column = j-th beta

###########Analysis of the data of Pluck & Ruales-Chieruzzi (2021)#######

library(sandwich) #for the sandwich estimator HC3

#IMPORTANT: Download the file
#Lexical tasks correlate with executive function tests.csv 
#from the website
#https://www.psycharchives.org/en/item/cc01f0e5-de42-4397-bf6b-7411a3853f7e
#then copy it in your working directory (getwd())

daten<-read.csv("Lexical tests in dementia and correlations with IQ.csv")
#separate Data in Training and Testing subsample (see Study 2 of Pluck & Ruales-Chieruzzi (2021))
dat_devel<-daten[daten$RandomGroup==1,]
dat_Test<-daten[daten$RandomGroup==2,]

#Looking for missing data or outliers
summary(dat_devel$WAT_Total)
summary(dat_devel$SpanLex_Final_version)
summary(dat_devel$SCIRT_Total)
dat_devel$WAT_Total[dat_devel$WAT_Total==99]<-NA #99 is a code for NA

#recompute results
Span_lm<-lm(dat_devel$IQ~dat_devel$SpanLex_Final_version)
WAT_lm<-lm(dat_devel$IQ~dat_devel$WAT_Total)
SCIRT_lm<-lm(dat_devel$IQ~dat_devel$SCIRT_Total)

summary(Span_lm)
summary(WAT_lm)
summary(SCIRT_lm)

#Results match with reported results of Pluck & Ruales-Chieruzzi (2021)

#compute HC-robust standard errors and CI

conf_HC<-function(x){#x is of type lm
  x$coef+sqrt(diag(vcovHC(x)))%*%t(qt(c(0.025,0.975),df=nrow(daten)-2))
}

#Standard errors (reported in Table 5, fourth column and 
#Supplementary Material II: Table 13, fourth column)

sqrt(diag(vcovHC(Span_lm)))
sqrt(diag(vcovHC(WAT_lm)))
sqrt(diag(vcovHC(SCIRT_lm)))

#Confidence Intervals (reported in Table 5, fifth column and 
#Supplementary Material II: Table 13, fifth column)

conf_HC(Span_lm)
conf_HC(WAT_lm)
conf_HC(SCIRT_lm)

##compute intervals for externally informed model

y<-dat_devel$IQ
XSpan<-dat_devel$SpanLex_Final_version
XWAT<-dat_devel$WAT_Total
XSC<-dat_devel$SCIRT_Total

#grip points = 10001, NA removed

#Analysis for [0.4,0.85]
interval_gmm(XSpan,y,c("RX1Y","EY"),c(0.4,100),c(0.85,100))
interval_gmm(XWAT[-which(is.na(XWAT))],y[-which(is.na(XWAT))],c("RX1Y","EY"),c(0.4,100),c(0.85,100))
interval_gmm(XSC,y,c("RX1Y","EY"),c(0.4,100),c(0.85,100))

#Analysis for [0.7,0.85]
interval_gmm(XSpan,y,c("RX1Y","EY"),c(0.7,100),c(0.85,100))
interval_gmm(XWAT[-which(is.na(XWAT))],y[-which(is.na(XWAT))],c("RX1Y","EY"),c(0.7,100),c(0.85,100))
interval_gmm(XSC,y,c("RX1Y","EY"),c(0.7,100),c(0.85,100))
